/* eyes.cpp
   Copyright (C) 1996, Abraham Flaxman (abie@kenlaw.com)

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */

// This source is based on code written by Benoit Schillings,
// the original heading comments follow.

//******************************************************************************
//
//	File:		main.cpp
//
//	Description:	3d hidden line main program
//
//	Written by:	Benoit Schillings
//
//	Copyright 1993, Be Incorporated
//
//	Change History:
//
//	7/31/93		bgs	new today
//	12/14/95	CKH	removing menu stuff
//
//******************************************************************************
#include <SupportDefs.h>
#include <Application.h>
#include <Alert.h>
#include <GraphicsDefs.h>
#include <stdlib.h>
#include <time.h>
#include "t3d.h"
#include <math.h>
#include "SmartSingleWindow.h"
#include "Preferences.h"
#define PREF_STRING	  "x-kenlaw-BeEyes"
#define PREF_BASE		"application/"

//J Watte's shared library stuff
Preferences gPrefs(PREF_STRING);
PreferenceSet gSettings(gPrefs, "settings", true);
/*------------------------------------------------------------*/

class MyApp : public BApplication {

public:
					MyApp (char *app_string);
			bool	QuitRequested ();
virtual 	void 	AboutRequested();
virtual 	void	MessageReceived(BMessage *msg);
};

/*------------------------------------------------------------*/

	MyApp::MyApp(char *app_string)
	  	  :BApplication (app_string)
{

}

void 
MyApp::MessageReceived(BMessage *msg)
{
	switch(msg->what)	{
		case 'abut':
			AboutRequested();
			break;
		default:
			inherited::MessageReceived(msg);
	}
	
}
void MyApp::AboutRequested()
{
	BAlert *alert = new BAlert("",
						"BeEyes\n Copyright 1997 Abraham Flaxman (abie@mit.edu)\n  Based on source by Benoit Shillings\nwhich bears a copyright 1993, Be Incorporated\n .",
						"OK");
						//,NULL,NULL,B_WIDTH_AS_USUAL,B_INFO_ALERT);
	alert->Go();
}

/*------------------------------------------------------------*/

bool MyApp::QuitRequested()
{
	be_app->PostMessage(B_QUIT_REQUESTED);
	return(TRUE);
}

/*------------------------------------------------------------*/
rgb_color make_color(uint8 r, uint8 g, uint8 b);

rgb_color	make_color(uint8 r, uint8 g, uint8 b)
{
	rgb_color	c;
	
	c.red = r;
	c.green = g;
	c.blue = b;
	return(c);
}

/*------------------------------------------------------------*/
const long CIR_GRAIN = 10;
const long SPH_GRAIN = 6;
void add_eye(TShape *a_shape,
				 float center_x,
				 float center_y,
				 float center_z,
				 float size);

void add_eye(TShape *a_shape,
				 float center_x,
				 float center_y,
				 float center_z,
				 float size)
{
	long first;
	float x, y, z, radius;
	
	first =
	a_shape->add_point(center_x, center_y, center_z+size);
	
	for( int j = 0; j < SPH_GRAIN; j++ ) {
		for( int i = 0; i < CIR_GRAIN; i++ ) {
			radius = size * (SPH_GRAIN - j) / SPH_GRAIN;
			x = radius * sin(2 * M_PI * i / CIR_GRAIN);
			y = radius * cos(2 * M_PI * i / CIR_GRAIN);
			z = sqrt( size*size - radius * radius );

			a_shape->add_point(center_x + x, center_y + y, center_z + z);
		}
	}
	
	first++;
	for( int i = 0; i < SPH_GRAIN - 1; i++ ) {
		rgb_color col;
		if( i < SPH_GRAIN - 3 )
			col = make_color(220, 220, 220);
		else
			col = make_color(65, 40, 255);

		for( int j = 0; j < CIR_GRAIN - 1; j++ ) {
			a_shape->add_poly(	first + i * CIR_GRAIN + j,
								first + i * CIR_GRAIN + j + 1,
								first + (i + 1) * CIR_GRAIN + j + 1,
								first + (i + 1) * CIR_GRAIN + j,
								col);
		}
		a_shape->add_poly(	first + i * CIR_GRAIN + CIR_GRAIN - 1,
							first + i * CIR_GRAIN,
							first + (i + 1) * CIR_GRAIN,
							first + (i + 1) * CIR_GRAIN + CIR_GRAIN - 1,
							col);
	}

	for( int j = 0; j < CIR_GRAIN - 1; j++ ) {
		a_shape->add_poly(	first + (SPH_GRAIN - 1) * CIR_GRAIN + j,
							first + (SPH_GRAIN - 1) * CIR_GRAIN + j + 1,
							first - 1,
							first - 1,
							make_color(32, 248, 32));
	}
}

/*------------------------------------------------------------*/

class	TMy3d : public T3d {

			TShape	*main_shape;
			float	alpha, beta, gamma;
			
			BPoint old_where;
			long idle_count;
			short bored_axis;
			
			thread_id mouse_watch;
			
public:
			bool	dying;

					TMy3d(BRect r,
						  long flags,
						  long size_x,
						  long size_y);
					~TMy3d();
			void	SetShape(TShape *a_shape);
	static	long	_mouse_watcher(void *data);		
			void	MouseWatcher();
	virtual void	MessageReceived(BMessage *an_event);
};

/*------------------------------------------------------------*/

TMy3d::TMy3d(BRect r, long flags, long size_x, long size_y) :
			T3d(r, flags, size_x, size_y)
{
	alpha = beta = gamma = 0;
		
	dying = false;
	
	idle_count = 0;
	
	mouse_watch = spawn_thread( _mouse_watcher, "mouse watcher",
			 B_NORMAL_PRIORITY, this );
	resume_thread( mouse_watch );
}

/*------------------------------------------------------------*/

TMy3d::~TMy3d()
{
	long e_v;

	dying = true;

// note to future generations:
// uncommenting the following
// line causes the program to fail
// in exiting.  why is that?

	//wait_for_thread( mouse_watch, &e_v );
	kill_thread(mouse_watch);
}

/*------------------------------------------------------------*/


void TMy3d::SetShape(TShape *a_shape)
{
	main_shape = a_shape;
}


/*------------------------------------------------------------*/

long TMy3d::_mouse_watcher(void *data)
{
	TMy3d *t = (TMy3d *)data;
	
	while( !t->dying ) {
		t->MouseWatcher();
		snooze(20.0 * 1000.0);
	}
	
	return 0;
}

/*------------------------------------------------------------*/

const float R_SQ = 400 * 400;

void	TMy3d::MouseWatcher()
{
	BPoint where;
	ulong buttons;
	
	if( Window( ) != NULL ) {
		Window( )->Lock( );

		BRect rect = Bounds( );
		
		GetMouse(&where, &buttons, FALSE);	

		if(where == old_where) {
			idle_count++;
			if( idle_count > 500 ) {
				if(bored_axis & 1)
					alpha += 0.05;
				if(bored_axis & 2)
					beta += 0.05;
				if(bored_axis & 4)
					gamma += 0.05;
				clear();
				main_shape->draw(alpha,
								 beta,
								 gamma,
								( rect.left + rect.right ) / 2, ( rect.top + rect.bottom ) / 2);
				Draw();
			} else {
				bored_axis = rand() % 4 + 1;
			}		
		} else {
			old_where = where;
			idle_count = 0;
	
			float x = where.x - ( rect.left + rect.right ) / 2;
			float y = where.y - ( rect.top + rect.bottom ) / 2;
	
			if( y * y > R_SQ )
				beta = M_PI_2;
			else
				beta = acos( ( R_SQ - y * y ) / sqrt( R_SQ * ( R_SQ - y * y ) ) );
			if( beta > M_PI_2 * 0.75 )
				beta = M_PI_2 * 0.75;
			if( y > 0 )
				beta *= -1;
	
			if( x * x > R_SQ )
				gamma = M_PI_2;
			else
				gamma = acos( ( R_SQ - x * x ) / sqrt( R_SQ * ( R_SQ - x * x ) ) );
			if( gamma > M_PI_2 * 0.75 )
				gamma = M_PI_2 * 0.75;
			if( x > 0 )
				gamma *= -1;
	
			clear();
			main_shape->draw(alpha,
							 beta,
							 gamma,
							( rect.left + rect.right ) / 2, ( rect.top + rect.bottom ) / 2);
			Draw();
		}
		Window( )->Unlock( );
	}
}

void	TMy3d::MessageReceived(BMessage *an_event)
{

	switch(an_event->what) {
		case	B_QUIT_REQUESTED:
			be_app->PostMessage(new BMessage(B_QUIT_REQUESTED));
			break;
		case 'abut':
			be_app->AboutRequested();
			break;
			
		default:
		inherited::MessageReceived(an_event);	
		MessageReceived(an_event);
	}
}

/*------------------------------------------------------------*/

class	TMainWindow : public SmartSingleWindow {
public:
	TMainWindow(BRect bound,
	   			char *name,
	   			window_type Wtype,
	   			long flags);
};

/*------------------------------------------------------------*/

TMainWindow::TMainWindow(BRect bound, char *name, 
		window_type type, long flags) :
		SmartSingleWindow(bound, name, type, flags)
{
	
	BRect a_rect(0,0,99,99);
	TMy3d *my_view = new TMy3d(a_rect, B_FOLLOW_NONE | B_WILL_DRAW, 100, 100);

	TShape *a_shape = new TShape(my_view);	
	add_eye(a_shape, 0, 0, 0, 100);

	my_view->SetShape(a_shape);
	AddChild(my_view);

	a_rect.Set(100,0,199,99);	
	my_view = new TMy3d(a_rect, B_FOLLOW_NONE | B_WILL_DRAW, 100, 100);

	a_shape = new TShape(my_view);	
	add_eye(a_shape, 0, 0, 0, 100);

	my_view->SetShape(a_shape);
	AddChild(my_view);
	
	Show();
}

/*------------------------------------------------------------*/

void main()
{
	char		buf[1024];
	
	sprintf(buf, "%s%s", PREF_BASE, PREF_STRING );
	
	set_thread_priority(find_thread(NULL), B_DISPLAY_PRIORITY);
	
	MyApp *my_app = new MyApp(buf);
  
  	
             
	BRect a_rect(100, 100, 300, 200);
	TMainWindow *a_window = new TMainWindow (a_rect, "BeEyes", B_TITLED_WINDOW,
					B_NOT_RESIZABLE|B_NOT_ZOOMABLE|B_WILL_ACCEPT_FIRST_CLICK);
	my_app->Run();
	//delete (my_app);
}
